# coding=utf-8

# import sys
# reload(sys)
# sys.setdefaultencoding('utf-8')
class BM(object):
    def log_error(self, *args, **kwargs):
        try:
            if args:
                print(args)
            if kwargs:
                print(kwargs)
        except:
            pass
BMOBJ = BM()

from .opends import OpenDS, OpenDSException


class Table(object):
    def __init__(self, ds, name, tb_id, schema=(), uniq_key=None, title=None):
        """
            the table object for bdp
        Args:
            ds (DS):
            name (str):
        """
        if not name:
            raise OpenDSException(u'Table name is required.')
        if not ds and not type(ds) != DS:
            raise OpenDSException(u'Ds is required.')
        self._name = name

        # remove duplicate field(dict) if exist in schema(list)
        self._schema = schema
        self._uniq_key = uniq_key
        self._ds = ds
        self._tb_id = tb_id
        if tb_id:
            if schema == ():
                pass
            else:
                self.modify_schema()
        else:
            self._tb_id = self._ds.opends.tb_create(
                None, self._ds.ds_id, name, self._schema, uniq_key, title).get('tb_id')

    def modify_schema(self):
        """
            modify the table schema, add field or delete failed.
        """
        schema_used = self._ds.opends.field_list(None, self._tb_id)
        schema_used_name = map(lambda field: field.get('name'), schema_used)
        schema_conf_name = map(lambda field: field.get('name'), self._schema)

        _need_add = filter(lambda field: field.get(
            'name') not in schema_used_name, self._schema)
        _need_del = filter(lambda field: field.get(
            'name') not in schema_conf_name, schema_used)

        for fd in _need_del:
            try:
                self._ds.opends.field_del(None, self._tb_id, fd.get('name'))
            except Exception as e:
                BMOBJ.log_error(
                    'Delete field `%s` failed and ignored, reason: %s' % (fd.get('name'), e))

        for fd in _need_add:
            try:
                self._ds.opends.field_add(
                    None, self._tb_id, fd.get('name'), fd.get('type'), 0)
            except Exception as e:
                BMOBJ.log_error(
                    'Add field `%s` failed and ignored, reason: %s' % (fd.get('name'), e))

    @property
    def name(self):
        return self._name

    @property
    def uniq_key(self):
        return self._uniq_key

    @property
    def ds(self):
        return self._ds

    @property
    def tb_id(self):
        return self._tb_id

    def __str__(self):
        return self._name

    def __repr__(self):
        return self._name

    def get_name(self):
        return self._name

    def get_id(self):
        return self._tb_id

    def insert_data_by_name(self, fields, data):
        """
            add data to current table
        :param fields: (list) eg: ["id", "name", "age"]
        :param data: (list) eg: [[1, "user", 30],[2, "user2", 31]]
        :return:
        """
        self._ds.opends.data_insert(None, self._tb_id, fields, data)
        return True

    def insert_data_by_id(self, fields, data):
        """
            add data to current table
        :param fields: (list) eg: ["id", "name", "age"]
        :param data: (list) eg: [[1, "user", 30],[2, "user2", 31]]
        :return:
        """
        self._ds.opends.data_insert_by_id(None, self._tb_id, fields, data)
        return True

    def insert_table_by_name(self, fields, data):
        """
            add data to current table
        :param fields: (list) eg: ["id", "name", "age"]
        :param data: (list) eg: [[1, "user", 30],[2, "user2", 31]]
        :return:
        """
        self._ds.opends.tb_insert(None, self._tb_id, fields, data)
        return True

    def insert_table_by_id(self, fields, data):
        """
            add data to current table
        :param fields: (list) eg: ["id", "name", "age"]
        :param data: (list) eg: [[1, "user", 30],[2, "user2", 31]]
        :return:
        """
        self._ds.opends.tb_insert_by_id(None, self._tb_id, fields, data)
        return True

    def bulk_delete(self, where):
        """
            bulk delete data from current table according to `where`
        :param where: (string) eg: "`id` > 3"
        :return:
        """
        self._ds.opends.data_bulkdelete(None, self._tb_id, where)
        return True
    
    def bulk_delete_yx(self, where):
        """
            bulk delete data from current table according to `where`
        :param where: (string) eg: "`id` > 3"
        :return:
        """
        result = self._ds.opends.data_bulkdelete_yx(None, self._tb_id, where)
        return result

    def delete_data_by_name(self, fields, data):
        """
            delete data according to field name
        :param fields: (list)
        :param data: (list)
        :return:
        """
        self._ds.opends.data_delete(None, self._tb_id, fields, data)
        return True

    def delete_data_by_id(self, fields, data):
        """
            delete data according to field id
        :param fields: (list)
        :param data: (list)
        :return:
        """
        self._ds.opends.data_delete_by_id(None, self._tb_id, fields, data)
        return True

    def update_data_by_name(self, fields, data):
        """
            update data according to field name
        :param fields: (list)
        :param data: (list)
        :return:
        """
        self._ds.opends.data_update(None, self._tb_id, fields, data)
        return True

    def update_data_by_id(self, fields, data):
        """
            update data according to field id
        :param fields: (list)
        :param data: (list)
        :return:
        """
        self._ds.opends.data_update_by_id(None, self._tb_id, fields, data)
        return True

    def commit(self):
        self._ds.opends.tb_commit(None, self._tb_id)
        return True

    def revert(self):
        self._ds.opends.tb_revert(None, self._tb_id)
        return True

    def clean(self):
        self._ds.opends.tb_clean(None, self._tb_id)

    def get_fields(self):
        return self._ds.opends.field_list(None, self._tb_id)

    def add_field(self, field_name, field_type, uniq_index=0, title=None):
        """
            add one field to current table
        :param field_name: (string)
        :param field_type:  (string) "string"/"number"/"date"
        :param uniq_index:  (number) 0/1
        :param title: (string)
        :return:
        """
        if not field_name:
            raise OpenDSException("field_name required")
        if field_type not in ["string", "number", "date"]:
            raise OpenDSException("wrong field_type")
        if uniq_index not in [0, 1]:
            raise OpenDSException("uniq_index can only be 0 or 1")
        self._ds.opends.field_add(
            None, self._tb_id, field_name, field_type, uniq_index, title)
        return True

    def modify_field(self, field_name, field_type, uniq_index=0, title=None):
        """
            modify one field of current table
        :param field_name: (string)
        :param field_type: (string) "string"/"number"/"date"
        :param uniq_index: (number) 0/1
        :param title: (string)
        :return:
        """
        if not field_name:
            raise OpenDSException("field_name required")
        if field_type not in ["string", "number", "date"]:
            raise OpenDSException("wrong field_type")
        if uniq_index not in [0, 1]:
            raise OpenDSException("uniq_index can only be 0 or 1")
        self._ds.opends.field_modify(
            None, self._tb_id, field_name, field_type, uniq_index, title)
        return True

    def delete_field(self, field_name):
        """
            delete one field of current table
        :param field_name: (string)
        :return:
        """
        if not field_name:
            raise OpenDSException("field name required")
        self._ds.opends.field_del(None, self._tb_id, field_name)
        return True

    def modify_table(self, alias_name):
        """
            modify the alias name of current table
        :param alias_name: (string)
        :return:
        """
        if not alias_name:
            raise OpenDSException("table alias name required")
        self._ds.opends.tb_name_modify(None, self._tb_id, alias_name)
        return True

    def preview(self):
        return self._ds.opends.tb_preview(None, self._tb_id)

    def get_info(self):
        return self._ds.opends.tb_info(None, self._tb_id)


class DS(object):
    def __init__(self, name, client):
        """
            the database object for bdp
        Args:
            name:
            client:
        """
        if not name:
            raise OpenDSException(u'DS name is required.')
        self._name = name
        self._opends = client.opends
        self._ds_id = None
        self.tables = {}
        for ds in self._opends.ds_list(None)['data_source']:
            if name == ds['name']:
                self._ds_id = ds['ds_id']
                for table in ds['tables']:
                    self.tables[table[0]] = Table(self, table[0], table[1])
                break

        if not self._ds_id:
            self._ds_id = self._opends.ds_create(None, name)["ds_id"]

    @property
    def opends(self):
        return self._opends

    @property
    def name(self):
        return self._name

    @property
    def ds_id(self):
        return self._ds_id

    @property
    def tb_ids(self):
        return [tb.tb_id for tb in self.tables.values()]

    def __str__(self):
        return self._name

    def __repr__(self):
        return self._name

    def get_name(self):
        return self._name

    def get_id(self):
        return self._ds_id

    def get_all_tables(self):
        return self.tables

    def create_table(self, name, schema, uniq_key=None, title=None):
        """
            create a table in current ds
        :param name: (string)
        :param schema: (list) eg: [{"comment": "", "name": "user", "type": "string"}, {"comment": "", "name": "id", "type": "number"}, {"comment": "", "name": "age", "type": "number"}]
        :param uniq_key: (list) eg: ["id"]
        :return:
        """
        if  isinstance(name, bytes):
            try:
                name = name.decode("utf-8")
            except Exception:
                raise OpenDSException("name can only be string")
        if not isinstance(schema, list):
            raise OpenDSException("schema can only be list")
        if len(schema):
            if isinstance(schema[0], list):
                schema = self._gen_schema(schema)
        table = Table(self, name, tb_id=None, schema=schema,
                      uniq_key=uniq_key, title=title)
        self.tables.update({name: table})
        return table

    def _gen_schema(self, schema):
        res = []
        for s in schema:
            if len(s) == 2:
                res.append({"name": s[0], "type": s[1]})
            elif len(s) == 3:
                res.append({"name": s[0], "type": s[1], "comment": s[2]})
            elif len(s) == 4:
                res.append({"name": s[0], "type": s[1],
                            "comment": s[2], "title": s[3]})

        return res

    def delete_table(self, name):
        """
            delete one table
        :param name: (string)
        :return:
        """
        if  isinstance(name, bytes):
            try:
                name = name.decode("utf-8")
            except Exception:
                raise OpenDSException("name can only be string")
        try:
            tb_id = self.get_table(name).tb_id
        except Exception:
            raise OpenDSException('table not exists： %s' % name)
        self._opends.tb_delete(None, tb_id)
        del self.tables[name]
        return True

    def get_table(self, name):
        if  isinstance(name, bytes):
            try:
                name = name.decode("utf-8")
            except Exception:
                raise OpenDSException("name can only be string")
        res = self.tables.get(name)
        if res is None:
            raise OpenDSException("table not exists: {name}".format(name=name))
        return res

    def destroy(self):
        self._opends.ds_delete(None, self._ds_id)
        return True

    def update_all(self):
        """
            Trigger cascading update
        """
        self._opends.tb_update(None, [tb.tb_id for tb in self.tables.values()])
        return True

    def update(self, tb_ids):
        self._opends.tb_update(None, tb_ids)
        return True


class BDPClient:
    def __init__(self, token=None, domain=None):
        if token is None:
            raise OpenDSException("no access_token,initial failed")
        if domain is None:
            raise OpenDSException("no domain,initial failed")
        self.token = token
        self._opends = OpenDS(token, domain)
        self._ds_dict = {}
        for ds in self._opends.ds_list(token)['data_source']:
            self._ds_dict.update({ds["name"]: DS(ds['name'], self)})

    def get_all_ds(self):
        return self._ds_dict

    def get_ds(self, name):
        if  isinstance(name, bytes):
            try:
                name = name.decode("utf-8")
            except Exception:
                raise OpenDSException("name can only be string")
        if self._ds_dict.get(name) is None:
            raise OpenDSException("ds not exists：%s" % name)
        return self._ds_dict.get(name)

    def create_ds(self, name=None):
        """
            create new data source
        :param name: (string)
        :return:
        """
        if name is None:
            raise OpenDSException("name can't be None")
        if  isinstance(name, bytes):
            try:
                name = name.decode("utf-8")
            except Exception:
                raise OpenDSException("name can only be string")
        if self._ds_dict.get(name) is not None:
            raise OpenDSException("%s ds already exists" % name)

        self._ds_dict.update({name: DS(name, self)})
        return self._ds_dict.get(name)

    def delete_ds(self, name=None):
        """
            delete one data source
        :param name: (string)
        :return:
        """
        if  isinstance(name, bytes):
            try:
                name = name.decode("utf-8")
            except Exception:
                raise OpenDSException("name can only be string")
        if not self.get_ds(name) is None:
            self.get_ds(name).destroy()
            del self._ds_dict[name]
        else:
            raise OpenDSException(u'ds not exists: `%s`' % name)

    @property
    def opends(self):
        return self._opends


# 暂时不开放以下


class SimpleClient:
    def __init__(self, token=None):
        if token is None:
            raise OpenDSException('token is required')
        self.token = token
        self._opends = OpenDS(token)
        self._ds_dict = {}
        dss = self._opends.ds_list(None)['data_source']
        for ds in dss:
            self._ds_dict.update({ds['name']: ds['ds_id']})

    def create_ds(self, name=None):
        if name is None:
            raise OpenDSException('ds name is reqiured')
        else:
            return self._opends.ds_create(None, name)['ds_id']

    def delete_ds(self, name):

        if name is None:
            raise OpenDSException('ds name is required')

        ds_id = self._ds_dict.get(name)
        if ds_id is None:
            raise OpenDSException('no ds names %s' % name)
        try:
            self._opends.ds_delete(None, ds_id)
        except Exception as e:
            raise OpenDSException(e)

    def list_ds(self):
        return self._opends.ds_list(None)['data_source']

    def create_table(self, ds_name, table_name, schema, uniq_key):
        ds_id = self._ds_dict.get(ds_name)
        if ds_id:
            try:
                return self._opends.tb_create(None, ds_id, table_name, schema, uniq_key)['tb_id']
            except Exception as e:
                raise OpenDSException(e)
        else:
            raise OpenDSException('ds %s is not exists')

    def insert_data(self, table_id, fields, data):
        try:
            self._opends.tb_insert(None, table_id, fields, data)
        except Exception as e:
            raise OpenDSException(e)

    def preview(self, table_id):
        try:
            return self._opends.tb_preview(None, table_id)
        except Exception as e:
            raise OpenDSException(e)

    def list_table(self, ds_name):
        ds_id = self._ds_dict.get(ds_name)
        if ds_id:
            return self._opends.tb_list(None, ds_id)
        else:
            raise OpenDSException('ds %s is not exists' % ds_name)

    def clean_table(self, table_id):
        try:
            self._opends.tb_clean(None, table_id)
        except Exception as e:
            raise OpenDSException(e)

    def delete_table(self, table_id):
        try:
            self._opends.tb_delete(None, table_id)
        except Exception as e:
            raise OpenDSException(e)

    def update_ds(self, ds_name):
        tables = self.list_table(ds_name)
        tb_ids = [t["tb_id"] for t in tables]
        self._opends.tb_update(None, tb_ids)

    def commit(self, table_id):
        self._opends.tb_commit(None, table_id)

    def modify_table_name(self, table_id, alias_name):
        self._opends.tb_name_modify(None, table_id, alias_name)

    def get_table_info(self, table_id):
        return self._opends.tb_info(None, table_id)

    def add_field(self, table_id, field_name, field_type, uniq_index, title=None):
        try:
            self._opends.field_add(
                None, table_id, field_name, field_type, uniq_index, title)
        except Exception as e:
            raise OpenDSException(e)

    def delete_field(self, table_id, field_name):
        try:
            self._opends.field_del(None, table_id, field_name)
        except Exception as e:
            raise OpenDSException(e)

    def list_field(self, table_id):
        return self._opends.field_list(None, table_id)

    def modify_field(self, table_id, field_name, field_type, uniq_index, title=None):
        try:
            self._opends.field_modify(
                None, table_id, field_name, field_type, uniq_index, title)
        except Exception as e:
            raise OpenDSException(e)

    def bulk_delete(self, table_id, where):
        try:
            self._opends.data_bulkdelete(None, table_id, where)
        except Exception as e:
            raise OpenDSException(e)

    def update_data(self, table_id, fields, data):
        try:
            self._opends.data_update(None, table_id, fields, data)
        except Exception as e:
            raise OpenDSException(e)

    def delete_data(self, table_id, fields, data):
        try:
            self._opends.data_delete(None, table_id, fields, data)
        except Exception as e:
            raise OpenDSException(e)



# coding=utf-8
from django.apps import apps
from tfduck.common.extendEncoder import DatetimeJSONEncoder
from django.utils.translation import ugettext_lazy as _
from django.db import models
from django.contrib import admin
from django.conf import settings
import ast
import hashlib
import json
import logging
import sys
import shutil
import time
import traceback
import requests
import os
import uuid
import arrow


class Et(Exception):
    """
    @author: yuanxiao
    @date: 2010-6-10
    @des: 自定义事物中的错误
    """

    def __init__(self, state=95635, v="default error", *args, **kwas):
        """
        """
        super(Et, self).__init__(*args, **kwas)
        self._msg = v
        self.state = state

    @property
    def msg(self):
        return json.dumps(self._msg)

    def getmsg(self):
        return self._msg


class CeleryRetryError(Et):
    pass


class BaseMethod(object):
    """
    @des:一些基础方法
    """

    def __init__(self):
        """
        @des:保持一些全局变量
        """
        self.logger_django = logging.getLogger('django')
        self.logger_dadian = logging.getLogger('dadian')
        self.http_api = Dj44HttpApi()

    def get_current_env(self):
        current_env = 'server'
        try:
            ddd = settings.DEBUG
        except:
            current_env = "local"
        return current_env

    def remove_file(self, file_path):
        """
        @des: 删除文件
        """
        try:
            os.remove(file_path)
        except:
            pass

    def remove_folder(self, folder_path):
        """
        @des: 删除文件夹
        """
        try:
            shutil.rmtree(folder_path)
        except:
            pass

    def get_file_path(self, path):
        """
        @des: 文件的真实路径
        """
        try:
            return os.path.join(settings.BASE_DIR, os.path.join("dags/sptasks/p_code", path))
        except:
            local_path = os.path.normpath(path)
            paths = local_path.split("/")
            if paths[0] == ".":
                real_path = "/".join(paths[2:])
            else:
                real_path = "/".join(paths[1:])
            return real_path

    def gen_local_unique_file(self, pre_path="", pre_name="", ext="csv"):
        """
        @des:生成本地文件唯一路径
        """
        # media_root = settings.MEDIA_ROOT
        media_root = "/mydata/media"
        base_dir = os.path.join(media_root, "docs", pre_path)
        if not os.path.exists(base_dir):
            os.makedirs(base_dir)
        real_name = "%s%s_%s.%s" % (pre_name, uuid.uuid1(
        ).hex, arrow.now().format("YYYYMMDD_HHmmSS"), ext)
        file_path = os.path.join(base_dir, real_name)
        return file_path

    def gen_local_unique_path(self, pre_path="", pre_name="", is_create=False):
        """
        @des:生成本地目录唯一路径
        """
        # media_root = settings.MEDIA_ROOT
        media_root = "/mydata/media"
        base_dir = os.path.join(media_root, "docs", pre_path)
        if not os.path.exists(base_dir):
            os.makedirs(base_dir)
        real_name = "%s%s_%s" % (pre_name, uuid.uuid1(
        ).hex, arrow.now().format("YYYYMMDD_HHmmSS"))
        folder_path = os.path.join(base_dir, real_name)
        if is_create:
            os.makedirs(folder_path)
        return folder_path

    def get_var(self, name):
        """
        @des: 获取变量
        """
        if self.get_current_env() == "local":
            return None
        else:
            VarConfig = apps.get_model("gconfig", 'VarConfig')
            try:
                obj = VarConfig.objects.get(name=name)
                return obj.contents
            except:
                return None

    def clog(self, ctx, *log_values):
        """
        @des: 向数据库中记录日志
        """
        try:
            pd_logs = [str(item) for item in log_values]
            pd_logs = "".join(pd_logs)
            if len(pd_logs) > 1024*1024*50:  # 大于5M
                log_values = ["log data too long"]
        except:
            pass
        if self.get_current_env() == "local":
            #
            real_values = []
            for log_value in log_values:
                try:
                    value = str(log_value)
                    real_values.append(value)
                except:
                    value = "log value must be string"
                    real_values.append(value)
            print(*real_values)
        else:
            task_type = ctx['task_type']
            trid = ctx['trid']
            index = ctx['index']
            #
            real_values = []
            for log_value in log_values:
                try:
                    value = str(log_value)
                    real_values.append(value)
                except:
                    value = "log value must be string"
                    real_values.append(value)

            Record = None
            if task_type == "dtask":
                Record = apps.get_model("dtask", 'RunRecord')
            elif task_type == "retask":
                Record = apps.get_model("retask", 'RETaskRecord')
            elif task_type == "sptask":
                Record = apps.get_model("sptask", 'DPTaskRecord')
            if Record is not None:
                try:
                    Record.objects.add_records(
                        lock_id=trid, task_index=index, recs=real_values)
                except Exception as e:
                    self.log_error("clog error ----- :")
                    self.logerr(e)
        # 防止过度写入日志
        time.sleep(0.2)

    def get_record_now(self, ctx={}, tz="UTC"):
        """
        @des: 获取执行任务记录的创建时间，方便dag里面取根据现在时间去执行part_date
        start = arrow.now(tz="Asia/Shanghai")
        @ return: datetime 带时区
        """
        if self.get_current_env() == "local":
            root_create_time = arrow.now(tz=tz)
        else:
            # 如果是查看任务，而不是任务记录，是不会有这两个参数的
            task_type = ctx.get('task_type', None)
            trid = ctx.get('trid', None)
            if trid is None:
                root_create_time = arrow.now(tz=tz)
            else:
                Record = None
                if task_type == "dtask":
                    Record = apps.get_model("dtask", 'RunRecord')
                elif task_type == "retask":
                    Record = apps.get_model("retask", 'RETaskRecord')
                elif task_type == "sptask":
                    Record = apps.get_model("sptask", 'DPTaskRecord')
                if Record is not None:
                    obj = Record.objects.get(id=trid)
                    #
                    root_create_time = obj.extras.get("root_create_time", None)
                    if root_create_time is None:  # 如果拿不到，那么obj肯定是根节点
                        root_create_time = arrow.get(obj.create_time).to(tz)
                    else:  # 读取extra的信息，即根节点
                        # root_create_time带有时区信息，可以直接get
                        root_create_time = arrow.get(root_create_time).to(tz)
        # 返回
        return root_create_time.datetime

    def jsonloads(self, data):
        """
        @des: 自动jsonload
        """
        try:
            data = json.loads(data)
        except Exception as e:
            try:
                data = ast.literal_eval(data)
            except Exception as e:
                raise Et(2, u"参数格式不正确")
        return data

    def jsondumps(self, data, mode=1, sort_keys=False):
        """
        @des:返回值dumps
        """
        #data = json.dumps(data, ensure_ascii=False, cls=DatetimeJSONEncoder)
        data = json.dumps(data, separators=(',', ':'), cls=DatetimeJSONEncoder)
        return data

    def text_tran_html(self, text):
        """
        @des:将文本转换为html,比如help_text
        """
        text = text.replace("\n", "<br/>")
        text = text.replace(" ", "&nbsp")
        return text

    def tran(self, text):
        """
        @des:翻译
        """
        tran_text = _("%s" % text)
        return tran_text

    # 日志记录
    def logerr(self, e):
        errorMeg = ''
        try:
            for file, lineno, function, text in traceback.extract_tb(sys.exc_info()[2]):
                errorMeg += '%s\n%s, in %s\n%s:                %s!' % (
                    str(e), file, function, lineno, text)
            self.log_error("error"+"*"*50)
            for error in errorMeg.split("\n"):
                self.log_error(error, "error")
            try:
                self.log_error(getattr(e, '_msg', 'exception'))
            except Exception as e1:
                self.log_error(getattr(e, 'msg', 'exception'))
        except Exception as e2:
            self.log_error(e2)

    def geterr(self, e):
        result = []
        errorMeg = ''
        try:
            for file, lineno, function, text in traceback.extract_tb(sys.exc_info()[2]):
                errorMeg += '%s\n%s, in %s\n%s:                %s!' % (
                    str(e), file, function, lineno, text)
            result.append("error"+"*"*50)
            for error in errorMeg.split("\n"):
                result.append(error)
                result.append('error')
            try:
                result.append(getattr(e, '_msg', 'exception'))
            except Exception as e1:
                result.append(getattr(e, 'msg', 'exception'))
        except Exception as e2:
            result.append(str(e2))
        return "\n".join(result)

    def get_log_str(self, *msgs):
        try:
            msg_str = " ".join([str(msg) for msg in msgs])
        except Exception as _:
            msg_str = msgs
        return msg_str

    def log_debug(self, *msgs):
        msg_str = self.get_log_str(*msgs)
        self.logger_django.debug(msg_str)

    def log_info(self, *msgs):
        msg_str = self.get_log_str(*msgs)
        self.logger_django.info(msg_str)

    def log_error(self, *msgs):
        msg_str = self.get_log_str(*msgs)
        self.logger_django.error(msg_str)

    def log_warning(self, *msgs):
        msg_str = self.get_log_str(*msgs)
        self.logger_django.warning(msg_str)
    # end--日志记录


class Dj44HttpApi(object):
    """
    @des: 外部api
    """

    def get_current_env(self):
        current_env = 'server'
        try:
            ddd = settings.DEBUG
        except:
            current_env = "local"
        return current_env

    def get_host_name(self):
        if self.get_current_env() == "local":
            return {'s': 13, 'msg': 'local'}
        else:
            GConfig = apps.get_model("gconfig", 'GConfig')
            try:
                obj = GConfig.objects.get(name='host_name')
                host_name = obj.contents['value']
            except:
                return {'s': 13, 'msg': 'host_name error'}
        return {'s': 1, 'v': host_name}

    # start----tfduck的外部接口调用
    def run_sptasks_by_taskname(self, user_token, task_names):
        """
        @des: http外部接口运行非定时的任务
        一般用于多个工程的上游依赖，A->B
        当A工程执行完毕后, 执行B工程的所有任务
        @param user_token[必填]: tfduck的用户token
               task_names[必填]: 运行的sptask任务(不是任务工程，也不是任务记录)的全名的列表, 比如 ["同步任务_3",...]
        @return: 
               成功---{"s":1,  'value':[任务名称]}
               失败---{"s: 非1, 'msg':'失败原因'}
        @example:
        ######
        from tfduck.common.defines import BMOBJ
        BMOBJ.http_api.run_sptasks_by_taskname('xxxxx', ['helloworld_end_tf_2_0'])
        ######
        """
        hresult = self.get_host_name()
        if hresult['s'] == 1:
            host_name = hresult['v']
        else:
            return hresult
        #
        url = os.path.join(host_name, 'run_sptasks_by_taskname')
        data = {
            'user_token': user_token,
            'task_names': json.dumps(task_names),
        }
        res = requests.post(url, data=data, timeout=(5, 30))
        result = res.json()
        return result

    def run_sptasks_by_projectname(self, user_token, project_names):
        """
        @des: http外部接口运行非定时的任务
        一般用于多个工程的上游依赖，A->B
        当A工程执行完毕后, 执行B工程的所有任务
        @param user_token   [必填]: tfduck的用户token
               project_names[必填]: 运行的sptask任务工程的名称(不是任务，也不是任务记录)的全名的列表, 比如 ["同步任务工程名",...]
        @return: 
               成功---{"s":1,  'value':[任务名称]}
               失败---{"s: 非1, 'msg':'失败原因'}
        @example:
        ######
        from tfduck.common.defines import BMOBJ
        BMOBJ.http_api.run_sptasks_by_projectname('xxxxx', ['helloworld_end_tf_2_0'])
        ######
        """
        hresult = self.get_host_name()
        if hresult['s'] == 1:
            host_name = hresult['v']
        else:
            return hresult
        #
        url = os.path.join(host_name, 'run_sptasks_by_projectname')
        data = {
            'user_token': user_token,
            'project_names': json.dumps(project_names)
        }
        res = requests.post(url, data=data, timeout=(5, 30))
        result = res.json()
        return result

    def get_sptasks_info_by_taskname(self, user_token, task_names, start_time, end_time):
        """
        @des: http外部接口查询任务的信息，根据任务名称查询
        一般用于多个工程的上游依赖，A->B
        当A工程执行完毕后, 执行B工程的所有任务
        @param user_token[必填]: tfduck的用户token
               task_names[必填]: 运行的sptask任务(不是任务工程，也不是任务记录)的全名的列表, 比如 ["同步任务_3",...]
               start_time[选填]: 任务的create_time的查询时间范围的开始时间，格式YYYY-MM-DD HH:mm:ss 例如 2022-02-15 14:23:44
               end_time  [选填]: 任务的create_time的查询时间范围的结束时间，格式YYYY-MM-DD HH:mm:ss 例如 2022-02-15 18:23:44
        @return: 
               成功---{"s":1,  'value':[{'task__name': '任务名称', 'task__id': 109, 'id': 78, 'state': 3, 'revoke_state': 1},...]}
               失败---{"s: 非1, 'msg':'失败原因'}
        @example:
        ######
        from tfduck.common.defines import BMOBJ
        BMOBJ.http_api.get_sptasks_info_by_taskname('xxxxx', ['helloworld_end_tf_2_0'], '2022-02-15 14:23:44', '2022-02-15 18:23:44')
        ######
        """
        hresult = self.get_host_name()
        if hresult['s'] == 1:
            host_name = hresult['v']
        else:
            return hresult
        #
        url = os.path.join(host_name, 'get_sptasks_info_by_taskname')
        data = {
            'user_token': user_token,
            'task_names': json.dumps(task_names),
            'start_time': start_time,
            'end_time': end_time,
        }
        res = requests.post(url, data=data, timeout=(5, 30))
        result = res.json()
        return result

    def get_sptasks_info_by_projectname(self, user_token, project_names, start_time, end_time):
        """
        @des: http外部接口查询任务的信息，根据任务工程名称查询
        一般用于多个工程的上游依赖，A->B
        当A工程执行完毕后, 执行B工程的所有任务
        @param user_token[必填]: tfduck的用户token
               project_names[必填]: 运行的sptask任务(不是任务工程，也不是任务记录)的全名的列表, 比如 ["同步任务_3",...]
               start_time[选填]: 任务的create_time的查询时间范围的开始时间，格式YYYY-MM-DD HH:mm:ss 例如 2022-02-15 14:23:44
               end_time  [选填]: 任务的create_time的查询时间范围的结束时间，格式YYYY-MM-DD HH:mm:ss 例如 2022-02-15 18:23:44
        @return: 
               成功---{"s":1,  'value':[{'task__name': '任务名称', 'task__id': 109, 'id': 78, 'state': 3, 'revoke_state': 1},...]}
               失败---{"s: 非1, 'msg':'失败原因'}
        @example:
        ######
        from tfduck.common.defines import BMOBJ
        BMOBJ.http_api.get_sptasks_info_by_projectname('xxxxx', ['helloworld_end_tf_2_0'], '2022-02-15 14:23:44', '2022-02-15 18:23:44')
        ######
        """
        hresult = self.get_host_name()
        if hresult['s'] == 1:
            host_name = hresult['v']
        else:
            return hresult
        #
        url = os.path.join(host_name, 'get_sptasks_info_by_projectname')
        data = {
            'user_token': user_token,
            'project_names': json.dumps(project_names),
            'start_time': start_time,
            'end_time': end_time,
        }
        res = requests.post(url, data=data, timeout=(5, 30))
        result = res.json()
        return result

    # end----tfduck的外部接口调用


BMOBJ = BaseMethod()



"""
未加密的tfduck/setup.py的代码
"""
import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="tfduck-bsd",
    version="0.11.7",
    author="yuanxiao",
    author_email="yuan6785@163.com",
    description="A small example package",
    long_description=long_description,
    long_description_content_type="text/markdown",
    # url="https://github.com/pypa/sampleproject",
    packages=setuptools.find_packages(),
    # 需要安装的依赖
    install_requires=[
         'requests>=2.20.0',
         'django==2.2.12',
         'oss2==2.5.0',
         'ThinkingDataSdk==1.6.2', #1.1.14, 1.6.2
         'kubernetes==12.0.1',
         'boto3==1.18.36'
    ],
    python_requires=">=3.5",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    scripts=['bin/tfduck'],
)
